package prime.toolbox;
import java.util.ArrayList;

import prime.toolbox.xml.reader.ReaderException;

/**
 * <code>ObjectCollection</code> class collects the
 * definition of objects read by the XML reader
 *  
 * @author      Pavel Boytchev
 * @version     1.3, 2006-10-10
 */
public class ObjectCollection {
	private ArrayList<ObjectDef> objectDefs = new ArrayList<ObjectDef>();

	/**
     * Adds a new object to the collection of objects.
     *
     * @param	objectDef	the object to be added to this collection
     */
	public void addObject( ObjectDef objectDef ) {
		objectDefs.add( objectDef );	
	}

	/**
     * Calculates the number of objects in this collection. 
     *
     * @return	the number of objects
     */
    public Integer size() {
        return objectDefs.size();
    }
    
	/**
     * Retrieves a specific object definition from the collection knowing its index. 
     *
     * @param	idx		the 0-based index of the object definition.
     * @return			the requested object definition or <code>null</code>
     * 					if the index is out of range
     */
    public ObjectDef getObjectDef( Integer idx ) {
   		if (idx>=size() || idx<0) return null;
        return objectDefs.get(idx);
    }
    
	/**
     * Retrieves a specific object definition from the collection knowing its name. 
     *
     * @param	name	the name of the object definition.
     * @return			the requested object definition or <code>null</code>
     * 					if the name is not found
     */
    public ObjectDef getObjectDef( String name ) {
   		for (int idx=0; idx<size(); idx++)
   		{
   			if (getObjectDef(idx).getName().equals(name))
   				return getObjectDef(idx);
   		}
   		return null;
    }
    
	/**
     * Converts this object into a string. 
     *
     * @return			the string representation of this object
     */
    public String stringOf() {
        String result = "";
        for (Integer idx=0; idx<size(); idx++) {
            result += getObjectDef(idx).stringOf()+"\n\n";
        }
        return result;
	}
    
	/**
     * Determines whether an object definition with a specific name
     * exists among all object definitions. 
     *
     * @param	name	the name of the object definition
     * @return			<code>true</code> if such an object exists,
     * 					<code>false</code> otherwise
     */
    public Boolean hasObjectDef( String name ) {
        for (Integer idx=0; idx<size(); idx++) {
            if (getObjectDef(idx).getName().equals(name)) return true;
        }
        return false;
    }
    
	/**
     * Runs consistency checks over the object definitions. Throws an
     * exception if:
     * <ul>
     * 		<li>a property of an object references an object which
     * 			is not present. A property can reference an object only through
     * 			its <code>type</code> -- when a property value is an object, then
     * 			the type of the property is the name of the object.</li>
     * 		<li>there is an object without any properties</li>
     * </ul>
     *
     * @throws	ReaderException
     */
    public void consistencyChecks() throws ReaderException {
    	// checks whether every used object name is actually defined
        for (Integer objIdx=0; objIdx<objectDefs.size(); objIdx++) {
        	ObjectDef objectDef = getObjectDef(objIdx);
        	
        	// check 1: references to undefined object
            for (Integer propIdx=0; propIdx<objectDef.size(); propIdx++) {
            	PropertyDef propertyDef = objectDef.getPropertyDef(propIdx);
               	if (!propertyDef.getType().equals("date")
               			&& !propertyDef.getType().equals("long")
               			&& !propertyDef.getType().equals("string")
               			&& !propertyDef.getType().equals("float")
               			&& !propertyDef.getType().equals("boolean")
               			&& !propertyDef.getType().equals("binary")
               			&& !propertyDef.getType().equals("kpi")
               			
               			&& !propertyDef.getType().equals("double")
               			&& !propertyDef.getType().equals("int")
               			
               			&& !hasObjectDef(propertyDef.getType()))
               	{
               		throw new ReaderException("Object '"+objectDef.getName()+"' has property '"+propertyDef.getName()+"' of unknown type '"+propertyDef.getType()+"'");
            	} //if
               	
               	if (propertyDef.hasReference()){
               		if (!hasObjectDef(propertyDef.getReference()))
               			throw new ReaderException("Object '"+objectDef.getName()+"' has property '"+propertyDef.getName()+"' of unknown reference '"+propertyDef.getReference()+"'");
               	}
            } // for propIdx

           	// check 2: object without properties
            if( objectDef.size()==0 )
           	{
           		throw new ReaderException("Object '"+objectDef.getName()+"' has no properties");
        	} //if

        } // for objIdx
    }
    
    public boolean isObjectInTOM(String objectName){
    	for (ObjectDef objDef : this.objectDefs){
    		if (objDef.getName().equals(objectName))
    			return true;
    	}
    	return false;
    }
}
