package prime.toolbox;

import java.util.ArrayList;

//import prime.net.dataobjects.IData;
import prime.game.dataobjects.Data;
import prime.toolbox.scomm.SComm;

/**
 * <code>ObjectData</code> class contains the
 * properties of a single object definition and
 * the values extracted from the server.
 *  
 * @author      Pavel Boytchev
 * @version     2.0, 2007-02-26
 */

public class ObjectData {
	private String civilName;
	private ObjectDef objectDef;
	private ArrayList<Object> objectData;
	private Object serverObjectReference;

	/**
     * Calculates the number of properties in this collection. 
     *
     * @return	the number of objects
     */
    public Integer size() {
        return objectDef.size();
    }
    
	/**
     * Finds whether a given property has single or multiple data 
     *
     * @param	idx		the 0-based index of the property
     * @return			<code>true</code> if the property is a sequence,
     * 					<code>false</code> if not, or if the index is
     * 					out of range
     */
    public boolean isSequence( Integer idx ) {
   		if (idx>=size() || idx<0) return false;
        return( getPropertyDef(idx).getSeq()!=null );
    }

    public static boolean isSequence ( PropertyDef prop ){
    	return ( prop.getSeq() != null );
    }

    /**
     * Finds whether a given property is an object 
     *
     * Note that you should explicitly check if  a property was not a 
     * structure by using the <code>isStruct</code> method even if you receive,
     * true after using this method. This is because structure are also defined
     * in TOM, together with Data objects.
 	 *
     * @param	idx		the 0-based index of the property
     * @return			<code>true</code> if the property is an object,
     * 					<code>false</code> if not, or if the index is
     * 					out of range
     */
    public boolean isObject( Integer idx ) {
   		if (idx>=size() || idx<0) return false;
   		String type = getPropertyDef(idx).getType();
        return(    !type.equals("int")
        		&& !type.equals("date")
       			&& !type.equals("long")
       			&& !type.equals("string")
       			&& !type.equals("float")
       			&& !type.equals("boolean")
       			
       			&& !type.equals("double")
       			&& !type.equals("int")
       			
//       			&& !type.equals("points")
//       			&& !type.equals("point")
//       			&& !type.equals("kpi")
			    && !type.equals("binary") );
    }

    /**
     * Finds whether a given property is an object 
     *
     * Note that you should explicitly check if an a property was not a 
     * structure by using the <code>isStruct</code> method even if you receive,
     * <code>true</code> after using this method. This is because structures are
     * also defined in TOM, together with Data objects.
 	 *
     * @param	type	the name of the property, as defined in TOM
     * @return			<code>true</code> if the property is an object,
     * 					or <code>false</code> if not.
     */
    public static boolean isObject( String type ) {
        return(    !type.equals("int")
        		&& !type.equals("date")
       			&& !type.equals("long")
       			&& !type.equals("string")
       			&& !type.equals("float")
       			&& !type.equals("boolean")       			
       			&& !type.equals("double")
       			&& !type.equals("int")    
			    && !type.equals("binary") );
    }
    
    /**
     * Note that you should explicitly check if an a property was not a 
     * structure even if you get <code>true</code>, when using either of the
     * <code>isObject</code> methods. This is because structures are also
     * defined in TOM, together with Data objects.
     * 
     * @param prop
     * @return
     */
    public static int isStruct ( PropertyDef prop ) {
		String objectName = prop.getType();
		if ( objectName.equals( "ISellable" ) ) return 0;
		ObjectDef def = SComm.getObjects().getObjectDef( objectName );
		if( def == null )
			return -1;
		try {
			Class c = Class.forName( def.getClassName() );
			if ( c.getDeclaredMethods().length == 0 )
				return 1;
			else
				return 0;
		} catch (ClassNotFoundException e) {
			System.err.println( "ObjectData.isStruct: Class " + def.getClassName() +
					" not found exception" );
			return -2;
//			e.printStackTrace();
		}
	}     

    
	/**
     * Retrieves a specific property definition from this object. 
     *
     * @param	idx		the 0-based index of the property
     * @return			the requested property or <code>null</code>
     * 					if the index is out of range
     */
    public PropertyDef getPropertyDef( Integer idx ) {
   		if (idx>=size() || idx<0) return null;
        return objectDef.getPropertyDef(idx);
    }

    /**
     * Retrieves property data from this object. 
     *
     * @param	idx		the 0-based index of the property
     * @return			the requested property or <code>null</code>
     * 					if the index is out of range or the property
     * 					is a sequence
     */
    public String getPropertyData( Integer idx ) {
   		if (idx>=size() || idx<0) return null;
   		if (isSequence(idx)) return null;

   		try {
   			if (objectData.get(idx) !=  null)
   				return objectData.get(idx).toString();
   			else {   				
//   				System.err.println("ObjectData.getPropertyData: No property data");
   				return null;
   			}
   		} catch ( IndexOutOfBoundsException e ){
  			printOutOfBoundsErrMessage( e );
   			return null;
   		}
    }
    
    public Object getPropertyIData( Integer idx ) {
   		if (idx>=size() || idx<0) return null;
   		if (isSequence(idx)) return null;
	
   		try {
   			return objectData.get(idx);
		} catch ( IndexOutOfBoundsException e ){
  			printOutOfBoundsErrMessage( e );
   			return null;
   		}
    }
    
    public boolean setPropertyIData( Integer idx, Data data ) {
//   		if (idx>=size() || idx<0) return false;
//   		if (isSequence(idx)) return false;
//   		
//   		try {
//   			objectData.set( idx,  data );
//		} catch ( IndexOutOfBoundsException e ){
//  			printOutOfBoundsErrMessage( e );
//   			return false;
//		}
//		
   		return true;
    }

    /**
     * Retrieves property sequence data from this object. 
     *
     * @param	idx		the 0-based index of the property
     * @return			the requested property or <code>null</code>
     * 					if the index is out of range or the property
     * 					is not a sequence
     */
    public ArrayList getPropertySequData( Integer idx ) {
   		if (idx>=size() || idx<0) return null;
   		if (!isSequence(idx)) return null;

   		try {
   			Object[] data = ( Object[] )objectData.get( idx );

   			if ( data == null ) return null;

   	   		ArrayList<Object> al = new ArrayList<Object>();
   	   		for ( Object obj : data )
   	   			al.add(obj);

   	   		return al;
		} catch ( IndexOutOfBoundsException e ){
  			printOutOfBoundsErrMessage( e );
			return null;
		} catch ( ClassCastException e) {
			e.printStackTrace();
			return null;
		}
    }

    @Deprecated
    public ArrayList getPropertySeqData( Integer idx ) {
   		if (idx>=size() || idx<0) return null;
   		if (!isSequence(idx)) return null;

   		try {
        	return (ArrayList)objectData.get( idx );
		} catch ( IndexOutOfBoundsException e ){
  			printOutOfBoundsErrMessage( e );
			return null;
		} catch ( ClassCastException e) {
			e.printStackTrace();
			return null;
		}

	}

    /**
     * Retrieves the name of this object 
     *
     * @return			the name of this object
     */
	public String getName() {
		return objectDef.getName();	
	}

	/**
     * Retrieves the civil name of this object.
     * By default the civil name is <code>null</code>. 
     *
     * @return			the civil name of this object
     */
	public String getCivilName() {
		return civilName;	
	}

	/**
     * Sets the civil name of this object.
     *
     * @param	name	the civil name of this object
     */
	public void setCivilName(String name) {
		civilName = name;	
	}

	/**
     * Converts this object into a string. Uses this format
     * <code>objname[propname1=value1 propname2=value2 ...]</code>
     * If a property is a sequence, then instead of
     * <code>propname=value</code> the generated string is
     * <code>propname=[value1 value2 ...]</code>.
     *
     * @return			the string representation of this object
     */
    public String stringOf() {
        String result = getName()+"[";
        for (int idx=0; idx<size(); idx++) {
        	if (idx>0) result += " ";
        	result += getPropertyDef(idx).getName()+"=";
        	if (isSequence(idx))
        	{
        		result += "[";
        		ArrayList data = getPropertySeqData(idx); 
        		for (int i=0; i<data.size(); i++)
        		{
                	if (i>0) result += " ";
        			result += ((String)data.get(i))+"";
        		}
        		result += "]";
        	}
        	else
        	{
                result += getPropertyData(idx);
        	}
        }
        result += "]";
        return result;
	}
    
    public String getClassName() {
    	return objectDef.getClassName();
    }
    
    public String getPackageName() {
    	String className = objectDef.getClassName();
    	if ( className.equals( "prime.game.simulation.OPLogistics" ) )
    		return "prime.net.dataobjects.distribution.";
    
    	int lastPoint = className.lastIndexOf( '.' );
    	return className.substring( 0, lastPoint + 1 ).replace( ".game.", ".net." );
    }
    
    public static String getPackageName( ObjectDef def ) {
    	String className = def.getClassName();
    	if ( className.equals( "prime.game.simulation.OPLogistics" ) )
    		return "prime.net.dataobjects.distribution.";

    	int lastPoint = className.lastIndexOf( '.' );
    	return className.substring( 0, lastPoint + 1 ).replace( ".game.", ".net." );
    }
    
 	public Object getServerObjectReference() {
		return serverObjectReference;
	}

	public void setServerObjectReference(Object serverObjectReference) {
		this.serverObjectReference = serverObjectReference;
	}

	/**
     * Constructs an <code>ObjectData</code> object containing
     * the definitions and data of properties. 
     *
     * @param objectDef		the definition of the properties
     * @param objectData	the values of the properties
     */
    public ObjectData (ObjectDef objectDef, ArrayList<Object> objectData) {
    	this.objectDef  = objectDef;
    	this.objectData = objectData;
    	this.civilName = null;
    	this.serverObjectReference = null;
    }
    
    public ObjectData ( ObjectDef objectDef, ArrayList<Object> objectData, Object objRef ) {
    	this.objectDef  = objectDef;
    	this.objectData = objectData;
    	this.civilName = null;
    	this.serverObjectReference = objRef;
    }
    private void printOutOfBoundsErrMessage(IndexOutOfBoundsException e) {
    		System.err.print( e.getMessage() );
			System.err.print( "ObjectData.getPropertyData: " );
			System.err.println( e.getMessage() );
//			e.printStackTrace();
    }
}
