package prime.toolbox;
import java.util.ArrayList;

import prime.toolbox.xml.reader.ReaderException;

/**
 * <code>ObjectDef</code> class contains the
 * properties of a single object definition.
 *  
 * @author      Pavel Boytchev
 * @version     1.3, 2006-10-16
 */
public class ObjectDef {
	private String name;
	private String className;
	private ArrayList<PropertyDef> propertyDefs = new ArrayList<PropertyDef>();

	/**
     * Adds a new property to the object deifnition.
     *
     * @param	propertyDef	the property to be added to this object
     */
	public void addProperty( PropertyDef propertyDef ) {
		propertyDefs.add( propertyDef );	
	}
	
	/**
     * Determines whether aproperty definition with a specific name
     * exists among all property definitions. 
     *
     * @param	name	the name of the property definition
     * @return			<code>true</code> if such a property exists,
     * 					<code>false</code> otherwise
     */
    public Boolean hasPropertyDef( String name ) {
        for (Integer idx=0; idx<size(); idx++) {
            if (propertyDefs.get(idx).getName().equals(name)) return true;
        }
        return false;
    }

    /**
     * Calculates the number of properties in this object. 
     *
     * @return	the number of properties
     */
	public Integer size() {
		return propertyDefs.size();	
	}

	/**
     * Retrieves the name of this object 
     *
     * @return			the name of this object
     */
	public String getName() {
		return name;	
	}

	/**
     * Retrieves the class name of this object 
     *
     * @return			the class name of this object
     */
	public String getClassName() {
		return className;	
	}

	/**
     * Retrieves a specific property from the object definition. 
     *
     * @param	idx		the 0-based index of the property.
     * @return			the requested property or <ode>null</code>
     * 					if the index is out of range
     */
	public PropertyDef getPropertyDef(Integer idx) {
   		if (idx>=size() || idx<0) return null;
		return propertyDefs.get(idx);	
	}

	/**
     * Retrieves a specific property from the object definition. 
     *
     * @param	name	the name of the property.
     * @return			the requested property or <ode>null</code>
     * 					if the name does not exist
     */
	public PropertyDef getPropertyDef(String name) {
        for (Integer idx=0; idx<size(); idx++) {
            if (propertyDefs.get(idx).getName().equals(name)) return propertyDefs.get(idx);
        }
		return null;	
	}

	/**
     * Converts this object into a string. 
     *
     * @return			the string representation of this object
     */
	public String stringOf() {
        String result = "object "+name;
        for (Integer idx=0; idx<propertyDefs.size(); idx++) {
            PropertyDef propertyDef = propertyDefs.get(idx);
            result += "\n    property " + propertyDef.stringOf();
        }
        return result;
	}
    
	/**
     * Constructs an empty object definition with a given name 
     *  i.e. the object does not contain any properties. Throws
     *  <code>ReaderException</code> exception if:
     *  <ul>
     *  	<li>the object name is redefined</li>
     *  	<li>the object name is empty</li>
     *  	<li>the object name is one of the reserved type: "date", "long", "string", "float" or "boolean"</li>
     *  </ul>
     *
     * @param	allObjects	the collection of objects to which this object will
	 *						be added simetime in the future. This parameter is
	 *						used to check for duplicates.
     * @param	name	the name of the object
     * @throws	ReaderException
     */
    public ObjectDef(ObjectCollection allObjects, String name, String className) throws ReaderException {
		if (allObjects.hasObjectDef(name))
			throw new ReaderException("Object '"+name+"' is redefined");

		if (name.length()<1)
			throw new ReaderException("Empty name of an object");

       	if (name.equals("date")
       			|| name.equals("long")
       			|| name.equals("string")
       			|| name.equals("float")
       			|| name.equals("boolean")
       			
       			|| name.equals("double")
       			|| name.equals("int")
       			
       	//		|| name.equals("points")
       	//		|| name.equals("point")
       			|| name.equals("kpi")
       			|| name.equals("binary") )
			throw new ReaderException("Invalid object name '"+name+"'");

       	this.name = name;
       	this.className = className;
    }
    
}
