package prime.toolbox;

import prime.toolbox.xml.reader.ReaderException;

/**
 * <code>PropertyDef</code> class defines a single
 * property of an object definition.
 *  
 * @author      Pavel Boytchev
 * @version     1.5, 2007-06-08
 */
public class PropertyDef {
	private String name;			// "<string>"
	private String type;			// "string", "float", "boolean", "long", "date", "binary", "<object-name>"
	private String seq;				// NULL, "0", "1"
	private String min;				// NULL, "<min-value>"
	private String max;				// NULL, "<max-value>"
	private String label;			// "<string>"
	private String description;		// NULL, "<string>"
	private String gui;				// "textbox", "listbox"
	private String guistyle;		// "normal", "bold"
	private Boolean civil;			// true, false
	private Boolean readonly;		// true, false
	
	private String reference;		// "string"
	
	/**
     * Retrieves the name of this property. The name is a string.
     *
     * @return			the name of this property
     */
	public String getName() {
		return name;
	}
	
	/**
     * Retrieves the type of this property. The type is a string
     * containing one of the words "date", "long", "string", "float"
     * "points", "binary", "point", "kpi" and "boolean", or it
     * contains the name of an object definition.
     *
     * @return			the type of this property
     */
	public String getType() {
		return type;
	}
	
	/**
     * Retrieves the sequencing limit of this property. If this limit
     * is 0 or 1, then it defines the minimal number of elements in this
     * property (i.e. the proeprty is a sequence). If the limit is
     * <code>null</code> then the property is not a sequence.
     *
     * @return			the sequence limit of this property
     */
	public String getSeq() {
		return seq;
	}
	
	/**
     * Retrieves the minimal value/length of this property. 
     * Minimal values are used by properties of types "long",
     * "float", "date". Minimal lengths are used by proeprties
     * of type "string".
     *
     * @return			the sequence limit of this property
     */
	public String getMin() {
		return min;
	}
	
	/**
     * Retrieves the maximal value/length of this property. 
     * Maximal values are used by properties of types "long",
     * "float", "date". Maximal lengths are used by proeprties
     * of type "string".
     *
     * @return			the max value/length of this property
     */
	public String getMax() {
		return max;
	}
	
	/**
     * Retrieves the label of this property. If the label is not
     * defined, then use <code>name</code> instead. 
     *
     * @return			the min value/length of this property
     */
	public String getLabel() {
		if (label!=null) return label;
		return name;
	}
	
	/**
     * Retrieves the description of this property. The description
     * is a line-long text which can be used in status lines and
     * hint popups. It can be <code>null</code>.
     *
     * @return			the description of this property
     */
	public String getDescription() {
		return description;
	}
	
	/**
     * Retrieves the name of the GUI control which will be used by 
     * editor of this property. If the control is not defined then
     * a textbox is assumed.
     *
     * @return			the name of the GUI control for this property
     */
	public String getGUI() {
		if (gui!=null) return gui;
		return "textbox";
	}
	
	/**
     * Retrieves the style of the GUI control which will be used by 
     * editor of this property. If the style is not defined then
     * a normal style is assumed.
     *
     * @return			the style of the GUI control for this property
     */
	public String getGUIStyle() {
		if (gui!=null) return guistyle;
		return "normal";
	}
	
	/**
     * Retrieve whether the property is a civil property. Civil properties
     * are displayed instead of object names when browsed by the user.
     *
     * @return			whether the property is civil
     */
	public Boolean isCivil() {
		return civil;
	}

	/**
     * Retrieve whether the property is a readonly property.
     *
     * @return			whether the property is readonly
     */
	public Boolean isReadonly() {
		return readonly;
	}
	
	/**
     * Retrieve whether the property is a sequence property.
     *
     * @return			whether the property is a sequence
     */
	public Boolean isSeq() {
		if (seq != null)
			return true;
		else
			return false;
	}

	/**
     * Converts this property into a string. 
     *
     * @return			the string representation of this property
     */
	public String stringOf() {
		String result = name+" ["+type;
		if (seq!=null) result+=" seq("+seq+")";
		if (label!=null) result+=" '"+label+"'";
		if (description!=null) result+=" '"+description+"'";
		if (min!=null) result+=" min="+min;
		if (max!=null) result+=" max="+max;
		if (gui!=null && !gui.equals("textbox")) result+=" "+gui;
		if (guistyle!=null && !guistyle.equals("normal")) result+=" "+guistyle;
		if (civil) result+=" civil";
		result+="]";
		return (result);
	}
	
	public String getReference(){
		return reference;
	}
	
	public boolean hasReference(){
		if (reference != null)
			return true;
		else 
			return false;			
	}

	/**
     * Constructs a property definition. Generates a <code>ReaderException</code> if:
     * <ul>
     * 		<li>the <code>name</code> of the property is empty</li>
     * 		<li>a property with such name already exists in the same object</li>
     * 		<li>the <code>seq</code> is not <code>null</code>, "0" or "1"</li>
     * 		<li>the <code>min</code> is not <code>null</code> and the property's type is not "long", "float", "date" or "string"</li>
     * 		<li>the <code>max</code> is not <code>null</code> and the property's type is not "long", "float", "date" or "string"</li>
     * 		<li>the <code>label</code> of the property is empty</li>
     * 		<li>the <code>gui</code> is not "textbox" or "listbox"</li>
     * 		<li>the <code>guistyle</code> is not "normal" or "bold"</li>
     * 		<li>the <code>seq</code> is not <code>null</code> and <code>civil</code> is true</li>
     * </ul>
     *
     * @param	parentObject	the object to which this property will be added
     * @param	name		the name of the object
     * @param	type		the type of this property
     * @param	seq			the sequence limit of this property
     * @param	min			the min value/length of this property
     * @param	max			the max value/length of this property
     * @param	label		the label of this property
     * @param	description	the description of this property
     * @param	gui			the name of the GUI control for this property
     * @param	guistyle	the style of the GUI control for this property
     * @param	civil		the civilianity of the property
     * @param	readonly	the readonly status of the property
     * @throws	ReaderException
     */
	public PropertyDef (ObjectDef parentObject,
						String name,
						String type,
						String seq,
						String min,
						String max,
						String label,
						String description,
						String gui,
						String guistyle,
						Boolean civil,
						Boolean readonly, 
						String reference) throws ReaderException
	{
		if (name.length()<1)
			throw new ReaderException("Empty name of a property in object '"+parentObject.getName()+"'");
		
		if (parentObject.hasPropertyDef(name))
			throw new ReaderException("Property '"+name+"' in object '"+parentObject.getName()+"' is redefined");

		if( seq!=null && !seq.equals("0") && !seq.equals("1") )
			throw new ReaderException("Invalid value of attribute seq=\""+seq+"\" in property '"+name+"' in object '"+parentObject.getName()+"'");
		
		if( !type.equals("long") && !type.equals("float") && !type.equals("date") && !type.equals("string") )
		{
			if( min!=null )
				throw new ReaderException("Invalid usage of tag <min>"+min+"</min> in property '"+name+"' in object '"+parentObject.getName()+"'");
			if( max!=null )
				throw new ReaderException("Invalid usage of tag <max>"+max+"</max> in property '"+name+"' in object '"+parentObject.getName()+"'");
		}
		
		if (label!=null && label.length()<1)
			throw new ReaderException("Empty contents of tag <label></label> in property '"+name+"' in object '"+parentObject.getName()+"'");

		if( gui!=null && !gui.equals("textbox") && !gui.equals("listbox") )
			throw new ReaderException("Invalid contents of tag <gui>"+gui+"</gui> in property '"+name+"' in object '"+parentObject.getName()+"'");
		
		if( guistyle!=null && !guistyle.equals("normal") && !guistyle.equals("bold") )
			throw new ReaderException("Invalid value of attribute style=\""+guistyle+"\" in property '"+name+"' in object '"+parentObject.getName()+"'");
			
		if( seq!=null && civil )
			throw new ReaderException("Invalid attribute civil=\""+seq+"\" in a sequence property '"+name+"' in object '"+parentObject.getName()+"'");
		
		
		
		this.name = name;
		this.type = type;
		this.seq = seq;
		this.min = min;
		this.max = max;
		this.label = label;
		this.description = description;
		this.gui = gui;
		this.guistyle = guistyle;
		this.civil = civil;
		this.readonly = readonly;
		this.reference = reference;
	}
}
