package prime.toolbox.gui;

import java.util.ArrayList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.TreeItem;

import prime.toolbox.ObjectData;
import prime.toolbox.ObjectDef;
import prime.toolbox.scomm.SComm;
import prime.toolbox.scomm.SCommException;

public class ObjectsView {
	
	/**
	 * <code>ObjectsView</code> class is used to create
	 * a <code>Tree</code> control that shows objects and
	 * their hierarchy.
	 * 
	 * @author      Aleksandar Dimov
	 * @version     2.0, 2007-03-06
	 */

	private static Shell	shell 		= null; 
	private static SComm	comm		= null;
	private static Tree 	tree		= null; 
	private static TreeItem	curItemSel	= null;
	
	/**
     * Constructs a new instance of this class, given
     * the composite and its display.
     */
	public ObjectsView(Shell cShell, Display cDisplay) {
		shell = cShell;
		tree = new Tree( shell, SWT.BORDER | SWT.MULTI 
				| SWT.H_SCROLL | SWT.V_SCROLL );
	}
	
	public ObjectsView(Shell cShell, Display cDisplay, SComm cComm) {
		shell = cShell;
		comm = cComm;
		tree = new Tree( shell, SWT.BORDER | SWT.MULTI 
				| SWT.H_SCROLL | SWT.V_SCROLL );
		final TreeColumn propertyName_col = new TreeColumn( tree, SWT.LEFT );
		final TreeColumn keyValue_col = new TreeColumn( tree, SWT.CENTER );

		propertyName_col.setText( "Object/Property Name" );
		propertyName_col.setWidth( 450 );

		keyValue_col.setText( "Property Value" );
		keyValue_col.setWidth( 130 );
	}
	
	

	/**
	 * @return Returns the comm.
	 */
	public static SComm getComm() {
		return comm;
	}

	/**
	 * @return the tree
	 */
	public static Tree getTree() {
		return tree;
	}

	public static TreeItem getCurItemSel() {
		return curItemSel;
	}

	/**
     * Creates the <code>Tree</code> control and attaches
     * the listeners for user actions to it.
     * 
     * @param	TOM_filename	The name of the XML Toolbox Object
     * 							Model file
     */
	
	public boolean makeTree( ){ 
		tree.setHeaderVisible( true );
		tree.setLinesVisible( true );
		
		if ( !comm.isConnected() ) return false; // not connected

		tree.addMouseListener( new PropertyChangeListener() );
		
		ObjectData vbe = comm.retrieveVBEData();
		TreeItem vbe_item = new TreeItem( tree, SWT.NONE );
		vbe_item.setText( new String[] { vbe.getName(), "" } );
		vbe_item.setData( vbe );
		@SuppressWarnings("unused")
		TreeItem newitem = new TreeItem( vbe_item, SWT.NONE );

		tree.addListener( SWT.Expand, new TreeListener() );

		tree.setFocus();
		vbe_item.setChecked( true );
		return true;
	}

	/**
     * Retreives object properties of an object sequence and attaches
     * them for visualization into the tree.
     * 
     * @param	theSubobjects	Array of the object properties 
     * @param	theItem			the parent <code>TreeItem</code>
     * 							of the object sequence
     */
	
	private void populateCollection(ArrayList theSubobjects, TreeItem theItem, String objectsName )
		throws SCommException {
		
		int size_subobjects = theSubobjects.size();
		theItem.setData( theSubobjects );

		for ( int j = 0; j < size_subobjects; j++ )
		{
			TreeItem subitem = new TreeItem( theItem, SWT.NONE );

			if ( ObjectData.isObject(objectsName) ) {
				Object subObject_data = theSubobjects.get( j );
				if ( subObject_data == null ) break;
				ObjectData subObject = null;
				if (objectsName.equals("IImage"))
					subObject = comm.retrieveImage(subObject_data);
				else 
					subObject = comm.retrieveObject( subObject_data, objectsName );
			
				subitem.setData( subObject );
				String civilName = subObject.getCivilName();

				if( civilName != null )
					subitem.setText( 0, civilName );
				else
					subitem.setText( 0, "Object number " + (j + 1) );

				TreeItem subsubitem = new TreeItem( subitem, SWT.NONE );
				subsubitem.setData( null );

			} else { // if collection of simple data properties
				subitem.setText( new String[] {
						"Value number " + (j + 1),
						theSubobjects.get(j).toString() } );
				subitem.setData( theSubobjects.get(j) );
			}

		} // endfor size_subobjects
		
		if ( size_subobjects == 0 )
			theItem.setText( 1, "Empty" );
		else {
			theItem.setText( 1, size_subobjects + " object(s)" );
		}

	} // populateCollection
	
	/**
     * Lists the properties of an object into the tree.
     *  
     * @param	theObject	Target object to vizualize
     * @param	theItem		the parent <code>TreeItem</code>
     * 						of the branch with the object properties 
     */
	private void populateObject(ObjectData theObject, TreeItem theItem) throws SCommException {

		for (int k = 0; k < theObject.size(); k++ )
		{
			TreeItem subitem = new TreeItem( theItem, SWT.NONE );
			subitem.setText( 0, theObject.getPropertyDef( k ).getLabel() );
			
			if ( theObject.isSequence( k ) ){
				ArrayList subobjects = theObject.getPropertySequData( k );
				if ( ( subobjects != null ) && ( subobjects.size() != 0 ) ){
					@SuppressWarnings("unused")
					TreeItem subsubitem = new TreeItem( subitem, SWT.NONE );
					String objectName = theObject.getPropertyDef(k).getType();
					populateCollection( subobjects, subitem, objectName );
				} else {
					// Dummy data in order to prevent subitem from disposal
					subitem.setData( new ArrayList(0) );
					subitem.setText( 1, "Empty" );
				}
				continue;

			} // endif isSequence
			
			if( theObject.isObject( k ) ) {
				String className = theObject.getPropertyDef(k).getType();
				ObjectData subobject = null; 

			//	if ( className.equals( "IImage" ) ){
			//		subitem.setData( new ObjectData( null, null ) );
			//		subitem.setText( 1, "IImage not supported yet" );
//					subobject = comm.getImageData( (IImage)theObject.getPropertyIData(k) );
			//	} else if ( className.equals( "ISellable" ) ){
				if ( className.equals( "ISellable" ) ){
					subitem.setData( new ObjectData( null, null ) );
					subitem.setText( 1, "ISellable object" );
//					subobject = comm.getSellableData( (ISellable)theObject.getPropertyIData(k) );
				}
				else if (className.equals( "IImage" )){
					Object propertyData = theObject.getPropertyIData( k );
					if ( propertyData != null )
						subobject = comm.retrieveImage(propertyData);
					
					if ( subobject == null ) { // Probably empty object
						// Dummy data in order to prevent subitem from disposal
						ObjectDef def = SComm.getObjects().getObjectDef( theObject.getPropertyDef( k ).getName() );
						subitem.setData( new ObjectData( def, null ) );
						subitem.setText( 1, "Empty" );
					} else {
						subitem.setData( subobject );
						@SuppressWarnings("unused")	TreeItem subsubitem = new TreeItem( subitem, SWT.NONE );
					}
				}				
				else {
					Object propertyData = theObject.getPropertyIData( k );
					if ( propertyData != null )
						subobject = comm.retrieveObject( propertyData, className );
				
				
					if ( subobject == null ) { // Probably empty object
						// Dummy data in order to prevent subitem from disposal
						ObjectDef def = SComm.getObjects().getObjectDef( theObject.getPropertyDef( k ).getName() );
						subitem.setData( new ObjectData( def, null ) );
						subitem.setText( 1, "Empty" );
					} else {
						subitem.setData( subobject );
						@SuppressWarnings("unused")	TreeItem subsubitem = new TreeItem( subitem, SWT.NONE );
					}
				}	
				continue;
			} //endif (isObject)

			subitem.setText( 0, theObject.getPropertyDef( k ).getLabel() );
			String itemPropertyText = theObject.getPropertyData( k ); 

			if ( theObject.getPropertyDef( k ).getType().equals( "date" ) )
				itemPropertyText = comm.date2String( new Long( itemPropertyText ).longValue() );


			if ( itemPropertyText != null )
				subitem.setText( 1, itemPropertyText );
			else
				subitem.setText( 1, "Error: Cannot get value" );

			// Sets the number of the property as subitem data
			subitem.setData( k );

		} // end of for object.size

	} // populateObject

	private class TreeListener implements Listener {

		/**
		 * 
		 */
		public TreeListener() {
			super();
		}

		/* (non-Javadoc)
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event event) {
			TreeItem item_sel = ( TreeItem ) event.item;

			TreeItem [] check_items = item_sel.getItems ();
			for (int i= 0; i<check_items.length; i++) {
				if (check_items[i].getData () != null) return;
				check_items[i].dispose ();
			}

			if ( item_sel.getData() == null ) return;	// DEBUG: Throw exception
			if ( item_sel.getData().getClass().getName()
					== "java.util.ArrayList" ) return;
			ObjectData object_item_sel= ( ObjectData )item_sel.getData();
			if ( object_item_sel == null ) return;	// DEBUG: Throw exception

			try {
				populateObject( object_item_sel, item_sel );
			} catch (SCommException e1) {
				e1.printStackTrace();
			}
		}

	} // End of TreeListener Class

	private class PropertyChangeListener implements MouseListener {

		/**
		 * 
		 */
		public PropertyChangeListener() {
			super();
		}

		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.MouseListener#mouseDoubleClick(org.eclipse.swt.events.MouseEvent)
		 */
		public void mouseDoubleClick( MouseEvent arg0 ) {

		}

		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.MouseListener#mouseDown(org.eclipse.swt.events.MouseEvent)
		 */
		public void mouseDown(MouseEvent event) {
			Point point = new Point ( event.x, event.y );

			curItemSel = tree.getItem( point );

		}

		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.MouseListener#mouseUp(org.eclipse.swt.events.MouseEvent)
		 */
		public void mouseUp(MouseEvent arg0) {

		}

	} // end of PropertyChangeListener class
	
} // end of ObjectsView class
