package prime.toolbox.scomm;

import java.text.*;
import java.util.*;

import prime.toolbox.*;
import prime.toolbox.xml.reader.Reader;
import prime.toolbox.xml.reader.ReaderException;

/**
 * <code>PseudoSComm</code> is a class shamming
 * the real <code>Comm</code> by generating
 * records with fake random data.
 * 
 * @author      Pavel Boytchev
 * @version     1.5, 2006-10-10
 */
public class PseudoSComm { 
	private ObjectCollection objects;
	private Random generator;
	private String dateFormat;

	/**
     * Generates a random date.
     *
     * @param prop		the complete property definition
     * @return			a random date matching the property deifnition
     */
	private String randomDate( PropertyDef prop ) throws SCommException
	{
		DateFormat df = new SimpleDateFormat(dateFormat);
		
		GregorianCalendar cal = new GregorianCalendar();
		GregorianCalendar min = new GregorianCalendar(); min.set(2000,0,0);
		GregorianCalendar max = new GregorianCalendar(); max.set(2010,12,31);

		try {
			if (prop.getMin()!=null) min.setTime(df.parse(prop.getMin()));
			if (prop.getMax()!=null) max.setTime(df.parse(prop.getMax()));
		}
		catch(ParseException pe)
		{
			throw new SCommException("Error parsing date");
		}

		long lmin=min.getTimeInMillis();
		long lmax=max.getTimeInMillis();

		Float resf=lmin+generator.nextFloat()*(lmax-lmin);
		cal.setTimeInMillis(resf.longValue());

		return df.format(cal.getTime());
	}
	
	/**
     * Generates a random string.
     *
     * @param prop		the complete property definition
     * @return			a random string matching the property deifnition
     */
	private String randomString( PropertyDef prop )
	{
		final String conso = "bcdgklmnprstv";
		final String vowlo = "aeio";
		int count = 4+generator.nextInt(8);
		String result = "";
		for( ; count>0; count--)
		{
			if (count%2==0)
				result += conso.charAt(1+generator.nextInt(conso.length()-1));
			else
				result += vowlo.charAt(1+generator.nextInt(vowlo.length()-1));
		}
		return result.substring(1,2).toUpperCase()+result.substring(2);
	}

	/**
     * Generates a random float.
     *
     * @param prop		the complete property definition
     * @return			a random float matching the property deifnition
     */
	private String randomFloat( PropertyDef prop )
	{
		float max = +1000000;
		float min = -1000000;
		if (prop.getMin()!=null) min=Float.parseFloat(prop.getMin());
		if (prop.getMax()!=null) max=Float.parseFloat(prop.getMax());
		Float result=generator.nextFloat();
		result = min+result*(max-min);
		return result.toString();
	}

	/**
     * Generates a random boolean.
     *
     * @param prop		the complete property definition
     * @return			a random boolean matching the property deifnition
     */
	private String randomBoolean( PropertyDef prop )
	{
		if (generator.nextBoolean())
			return "true";
		else
			return "false";
	}

	/**
     * Generates a random long.
     *
     * @param prop		the complete property definition
     * @return			a random long matching the property deifnition
     */
	private String randomLong( PropertyDef prop )
	{
		long max = +10000;
		long min = -10000;
		if (prop.getMin()!=null) min=Long.parseLong(prop.getMin());
		if (prop.getMax()!=null) max=Long.parseLong(prop.getMax());
		Long result=Math.abs(generator.nextLong());
		result = min+(result % (max-min));
		return result.toString();
	}

	/**
     * Generates a random binary.
     *
     * @param prop		the complete property definition
     * @return			a random binary
     */
	private String randomBinary( PropertyDef prop )
	{
		int i;
		int count = generator.nextInt(20);
		String result = "";

		for( i=0; i<=count; i++)
			result = result +  Integer.toHexString(generator.nextInt(256));
		return result;
	}

	/**
     * Generates a random points.
     *
     * @param prop		the complete property definition
     * @return			a random polygon points
     */
	private String randomPoints( PropertyDef prop )
	{
		int i;
		int count = 2*generator.nextInt(10);
		String result = " ";

		for( i=0; i<count; i++)
			result = result + generator.nextInt(100) + " ";

		return result;
	}

	/**
     * Generates a random KPI.
     *
     * @param prop		the complete property definition
     * @return			a random KPI
     */
	private String randomKPI( PropertyDef prop )
	{
		String result = "KPI" + generator.nextInt(100);

		return result;
	}

	/**
     * Generates a random point.
     *
     * @param prop		the complete property definition
     * @return			a random point
     */
	private String randomPoint( PropertyDef prop )
	{
		String result = " " + generator.nextInt(100) + " " + generator.nextInt(100) + " ";

		return result;
	}

	/**
     * Generates a random object.
     *
     * @param prop		the complete property definition
     * @return			a random object id matching the property deifnition
     */
	private String randomObject( PropertyDef prop )
	{
		Integer res = prop.getType().hashCode();
		res = 100*(Math.abs(res) % 1000)+generator.nextInt(100);
		return "@"+res.toString();
	}

	/**
     * Generates a random value.
     *
     * @param prop		the complete property definition
     * @return			a random value matching the property deifnition
     */
	private String randomValue( PropertyDef prop ) throws SCommException
	{
		if (prop.getType().equals("date"))
			return randomDate(prop);
		else if (prop.getType().equals("string"))
			return randomString(prop);
		else if (prop.getType().equals("float"))
			return randomFloat(prop);
		else if (prop.getType().equals("boolean"))
			return randomBoolean(prop);
		else if (prop.getType().equals("long"))
			return randomLong(prop);
		else if (prop.getType().equals("binary"))
			return randomBinary(prop);
		else if (prop.getType().equals("points"))
			return randomPoints(prop);
		else if (prop.getType().equals("point"))
			return randomPoint(prop);
		else if (prop.getType().equals("kpi"))
			return randomKPI(prop);
		else
			return randomObject(prop);
	}

	/**
     * Generates a <code>ObjectData</code> object containing
     * definition and the values of properties. The values are
     * generated, i.e. they are not real data retrieved by the
     * PRIME server.
     *
     * @param objectGlobalId	a fake but global ID of the object. 
     *							The global ID format is <code>@yyyzz</code> 
     * 							where <code>yyy</code> is the ID
     * 							of the object definition, and <code>zz</code>
     * 							is the local IS of the object
     * 
     * @return					an object containing property definitions taken from the
     * 							XML Reader and generated property values
     * 
     * @throws					SCommException	
     */
	public ObjectData retrieveObject( String objectGlobalId ) throws SCommException
	{
		String objectName=null;
		Long objectId;
		
		if (objectGlobalId.charAt(0)!='@')
			throw new SCommException("Invalid object global ID: '"+objectGlobalId+"' - does not start with @");
		objectGlobalId = objectGlobalId.substring(1);
		
		objectId = Long.parseLong(objectGlobalId.substring(objectGlobalId.length()-2));
		
		for (int i=0; i<objects.size(); i++)
		{
			Integer code = objects.getObjectDef(i).getName().hashCode();;
			code = Math.abs(code) % 1000;
			if (objectGlobalId.equals(""+(100*code+objectId)))
			{					
				objectName = objects.getObjectDef(i).getName();
				break;
			}
		}
		
		if (objectName==null)
			throw new SCommException("Invalid object global ID: '"+objectGlobalId+"' - object does not exist");
			
		//System.out.println("object "+objectGlobalId+" = "+objectName+"."+objectId);		
		return retrieveObject(objectName,objectId);
	}
	
	/**
     * Generates a <code>ObjectData</code> object containing
     * definition and the values of properties. The values are
     * generated, i.e. they are not real data retrieved by the
     * PRIME server.
     *
     * @param objectName	the name of the object to be generated
     * @param objectId		a fake local ID of the object. Used as a seed for
     * 						randomization. The same ID will always produce
     * 						the same random contents of the result.
     * 
     * @return				an object containing property definitions taken from the
     * 						XML Reader and generated property values
     * 
     * @throws				SCommException	
     */
	public ObjectData retrieveObject( String objectName, long objectId ) throws SCommException
	{
		ObjectData result;
		generator = new Random(objectId);
		for ( int i=0; generator.nextInt(5)<3; ) i=generator.nextInt(Math.abs(i)+1);
		
		ArrayList<Object> data = new ArrayList<Object>();
		
		ObjectDef def = objects.getObjectDef(objectName);
		if(def==null)
			throw new SCommException("Object not defined");
		
		// generating fake data
		for( int i=0; i<def.size(); i++ )
		{
			PropertyDef prop = def.getPropertyDef(i);

			if (prop.getSeq()==null)
				data.add(randomValue(prop));
			else {
				ArrayList<Object> seq = new ArrayList<Object>();
				int count = generator.nextInt(6);
				if (prop.getSeq().equals("1") && count==0) count=1;
				for( ; count>0; count-- ) seq.add(randomValue(prop));
				data.add(seq);
			}
		}
		
		// pack object data (without civil name)
		result = new ObjectData(def,data);
		
		// generate the civil name of the object
		String civilName = "";
		Boolean hasCivilName = false;
		for ( int i=0; i<result.size(); i++ )
		{
			PropertyDef prop = def.getPropertyDef(i);
			if (prop.isCivil())
			{
				if (result.isObject(i))
				{
					ObjectData temp = retrieveObject((String)data.get(i));
					civilName += temp.getCivilName()+" ";
				}
				else
					civilName += data.get(i)+" ";
				hasCivilName = true;
			}
		}
		
		if (hasCivilName) result.setCivilName(civilName.substring(0,civilName.length()-1));
		
		// the object data is finally ready
		return result;
	}
	
	/**
     * Constructs an <code>PseudoSComm</code> object. This objects
     * shams a Comm by returning random data as if there were
     * a real running PRIME server.
     *
     * @param filename		the file name of the XML definitions of objects
     */
	public PseudoSComm( String filename ) throws ReaderException
	{
		Reader xmlReader;
		xmlReader = new Reader(filename);
		objects = xmlReader.getObjects();
		dateFormat = xmlReader.getDateFormat();
	}
}
