package prime.toolbox.xml.reader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Enumeration;

import prime.toolbox.ObjectCollection;
import prime.toolbox.ObjectDef;
import prime.toolbox.PropertyDef;
import sjg.xml.*;

/**
 * <code>Reader</code> class loads the definition
 * of the objects in the Toolbox Object Model from an XML
 * file.
 *  
 * @author      Pavel Boytchev
 * @version     1.5, 2006-10-16
 */
public class Reader { 

	private ObjectCollection objects =  new ObjectCollection();
	private String dateFormat;
	private String version;
	
	private String getSingleData( Element tag, String name) {
		if( tag==null )
			return null;
		
		// try with attribute
    	if ( tag.hasAttribute(name))
			return (tag.getAttribute(name).getValue());
    	
		// try with element
		if ( tag.elements(name).hasMoreElements())
			return (((Element)tag.elements(name).nextElement()).getContents().toString().trim());
		
		// return the default value
		return (null);
    }

	private String getCompulsoryData( Element tag, String name) throws ReaderException {
		if (tag!=null)
		{
			// try with attribute
			if ( tag.hasAttribute(name))
				return (tag.getAttribute(name).getValue());
			// try with element
			if ( tag.elements(name).hasMoreElements())
				return (((Element)tag.elements(name).nextElement()).getContents().toString().trim());
		}
		{
			// return the default value
			throw new ReaderException("A <"+tag.getName()+"> tag without '"+name+"' attribute:\n"+tag.toString());
		}
    }

	/**
     * Retrieves a {@link ObjectCollection} object containing
     * the definitions of all objects read from the XML file.
     *
     * @return	the definition of objects
     * @see		ObjectCollection 
     */
	public ObjectCollection getObjects() {
		return objects;
	}
	
	/**
     * Retrieves the version of the XML file.
     * If version attribute does not exist then return "0.0"
     *
     * @return	the version
     */
	public String getVersion() {
		if (version==null)
			return "0.0";
		else
			return version;
	}

	/**
     * Retrieves the date format of the XML file.
     * This is the date pattern used for all dates in the XML file.
     * If date format attribute does not exist then return "yyyy-mm-dd"
     *
     * @return	the date format
     */
	public String getDateFormat() {
		if (dateFormat==null)
			return "yyyy-mm-dd";
		else
			return dateFormat;
	}

	/**
     * Initializes the Toolbox XML Reader, reads an XML file
     * defined by its filename, and checks its contents.
     * Throws and exception if:
     * <ul>
     *		<li> the input file is not found </li>
     *		<li> there is a duplication of object definition names </li> 
     * </ul>
     *
     * @param	filename	the name of the XML file
     * @throws				ReaderException
     */
	public Reader (String filename) throws ReaderException {
		try
		{
			FileInputStream fstream = new FileInputStream(filename);

			Document xmlDocument = Parser.parse(fstream); 
			Element xmlObjectModel = xmlDocument.getRoot();

			version = getSingleData(xmlObjectModel,"version");
			dateFormat = getSingleData(xmlObjectModel,"dateformat");

			if (!xmlObjectModel.getName().equals("objectmodel"))
				throw new ReaderException("Main tag is not <objectmodel>");

				
				for (Enumeration xmlObjects = xmlObjectModel.elements("object"); xmlObjects.hasMoreElements(); ) {

					Element xmlObject = (Element)xmlObjects.nextElement();
					ObjectDef objectDef = new ObjectDef(objects,getCompulsoryData(xmlObject,"name"),getCompulsoryData(xmlObject,"class"));
					
					for (Enumeration xmlProperties = xmlObject.elements("property"); xmlProperties.hasMoreElements(); ) {
						Element xmlProperty = (Element)xmlProperties.nextElement();
						Element gui = null;

						if ( xmlProperty.elements("gui").hasMoreElements())
							gui = ((Element)xmlProperty.elements("gui").nextElement());

						PropertyDef propertyDef = new PropertyDef(
								objectDef,
								getCompulsoryData(xmlProperty,"key"),
								getCompulsoryData(xmlProperty,"type"),
								getSingleData(xmlProperty,"seq"),
								getSingleData(xmlProperty,"min"),
								getSingleData(xmlProperty,"max"),
								getSingleData(xmlProperty,"label"),
								getSingleData(xmlProperty,"description"),
								getSingleData(xmlProperty,"gui"),
								getSingleData(gui,"style"),
								(xmlProperty.hasAttribute("civil")),
								(xmlProperty.hasAttribute("readonly")),
								getSingleData(xmlProperty,"reference")
							);
						objectDef.addProperty(propertyDef);
					} // for xmlProperties
					objects.addObject(objectDef);

				} // for xmlObjects
			objects.consistencyChecks();
		} 
		catch(FileNotFoundException fnfe)
		{
			throw new ReaderException("File not found");
		}		
	}
}
